#include "main.h"
#include "memory.h" 
#include "HID.h" 


void mandelbrot_3DS(int stackFlag);
s64 fxptMult(s64 a, s64 b);
void solidScreen(int size, int address);
void getInput();
void reset();
void push();
void pop();
void clearScreen();
void stats();

int rm, gm, bm, zoom, stackFlag = 0;
char *sf="by snailface";
s64 offx;
s64 offy;
s64 sx = 0x01EB85;
s64 sy = 0x022222;
s64 tx = 200;
s64 ty = 120;
s64 stack[32];


int main() {

    offx = -0x2000000;
    offy = -0x1000000;
    zoom = 0;
   
    
    while (1) {
        
        mandelbrot_3DS(0);
        getInput();
        
    }
 return 0;
   
}

void mandelbrot_3DS(int stackFlag) {
    clearScreen();
    const int MAX = 48;
    int offset3D = 0;
    int j;
    int lookupOS[]={0,1,2,3,4,5,6,7,8,7,6,5,4,3,2,1}; //3D offset lookup optimization
    int lookupCL[]={250,228,206,184,162,152,130,108,86};  //color lookup optimization
    sx = 0x01EB85;
    sy = 0x022222;


    //float sy=(2.0/240.0/zp);
    //float sx=(3.0/400.0/zp);
    for (j = 0; j < zoom; j++) {
        sx = fxptMult(sx, 0x800000);
        sy = fxptMult(sy, 0x800000);
    }


    if ((zoom > 0) && stackFlag == 0) {
        offx = offx + fxptMult((tx << 25) - 0xC8000000, sx);
        offy = offy + fxptMult((ty << 25) - 0x78000000, sy);
    } 
    stackFlag=0;
    stats();



    s64 x0 = 0;
    s64 y0 = 0;
    s64 x = 0;
    s64 y = 0;
    s64 temp = 0;
    s64 X = 0;
    s64 Y = 0;
    int i = 0;
    int fadeIn;

    s64 xx, yy;
    u8 r, g, b;
    r = g = b = 0;


    for (fadeIn = 8; fadeIn < 16; fadeIn++) {

        for (Y = 0; Y < 240; Y++) {

            for (X = fadeIn; X < 392; X +=8) {
             
                x = y = i = xx = yy = 0;

                x0 = (fxptMult(sx, X << 24)) + offx;
                y0 = (fxptMult(sy, Y << 24)) + offy;

                while (((xx + yy) < 0x4000000) && (i < MAX)) {
                    temp = (xx - yy) + x0;
                    y = fxptMult(0x2000000, fxptMult(x, y)) + y0;

                    x = temp;

                    xx = fxptMult(x, x);
                    yy = fxptMult(y, y);
                    i++;
                }
                
                //r = i << rm;
                //g = i << bm;
                //b = i*2;
                offset3D = lookupOS[i & 15];       
              
                r =lookupCL[offset3D];

                paintPixel3D(X, Y, r, 0, 0, offset3D);
                /*
                paint_pixel(X + offset3D, Y, r, g, Y, TOP_LEFT_FRAME2);
                paint_pixel(X - offset3D, Y, r, g, Y, TOP_RIGHT_FRAME1);
                paint_pixel(X - offset3D, Y, r, g, Y, TOP_RIGHT_FRAME2);
                /**/


            }

        }
    }

};

s64 fxptMult(s64 a, s64 b) {
    return (a * b) >> 24;
};


void solidScreen(int size, int address) {

    int *p = (int*) address;

    while (size--) {

        *(p++) = 0x0; //little endian, ha!
        *(p++) = 0x0;
        *(p++) = 0x0;

    } //FF0000FF 0000FF00 00FF0000 <- doens't work cause its not little endian lol

};

void reset() {
    zoom = 0;
    offx = -0x2000000;
    offy = -0x1000000;
};

void getInput() {
    int input = (read_word(HID));
    int z = 0, i = 0;
    int c = 255;



    while (1) {

        input = ~read_word(HID);

        //printInteger(input, 0, 10,c, c, c,SUB_FRAME1);
        //printInteger(input, 0, 10,c, c, c,SUB_FRAME2);
        //char *w = "Press";

        if ((input & BUTTON_A)) {
            push();
            break;
        }
        if ((input & BUTTON_B)) {

            pop();
        }
        if ((input & BUTTON_X)) {
            reset();
            break;
        }

        if ((input & BUTTON_UP)) {
            ty--;
        }


        if ((input & BUTTON_LEFT)) {
            tx--;
        }


        if ((input & BUTTON_DOWN)) {
            ty++;
        }


        if ((input & BUTTON_RIGHT)) {
            tx++;
        }
      
        if(tx<8)tx=8;
        if(tx>392)tx=392;
        if(ty<8)ty=8;
        if(ty>232)ty=232;
        
      


        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_LEFT_FRAME1);
        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_LEFT_FRAME2);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_RIGHT_FRAME1);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_RIGHT_FRAME2);
        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_RIGHT_FRAME1);
        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_RIGHT_FRAME2);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_LEFT_FRAME1);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_LEFT_FRAME2);

        wait(200);
        //drawBox(0,20,260,12,0,0,SUB_FRAME1);
        //drawBox(0,20,260,12,0,0,SUB_FRAME2);
        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_LEFT_FRAME1);
        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_LEFT_FRAME2);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_RIGHT_FRAME1);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_RIGHT_FRAME2);
        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_RIGHT_FRAME1);
        paint_line(tx - 5, ty, 11, 0, 0xFF, TOP_RIGHT_FRAME2);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_LEFT_FRAME1);
        paint_line(tx, ty + 5, 11, 1, 0xFF, TOP_LEFT_FRAME2); 
        //solidScreen(19200,  SUB_FRAME1);
        //solidScreen(19200,  SUB_FRAME2);
    }

};

void push() {
    int p=zoom*2;
    if (zoom < 16) {
        stack[p] = offx;
        p++; //workaround for random file size crashes
        stack[p] = offy;
    }
    zoom++;

};

void pop() {
    zoom--;
    if (zoom<0){
        zoom=0;
        return;
    }
    if (zoom>15){
        zoom=15;
    }
    
    offx = stack[zoom * 2];
    offy = stack[zoom * 2 + 1];
    mandelbrot_3DS(1);

};

void clearScreen(){
    solidScreen(24000, TOP_LEFT_FRAME1);
    solidScreen(24000, TOP_RIGHT_FRAME1);
    solidScreen(19200, SUB_FRAME1);
    solidScreen(24000, TOP_LEFT_FRAME2);
    solidScreen(24000, TOP_RIGHT_FRAME2);
    solidScreen(19200, SUB_FRAME2);
    
  
};
void stats(){
    char *title="Mandelbrot 3DS";
    char *zx="offstX";
    char *zy="offstY";
    char *px="scaleX";
    char *py="scaleY";
    char *ztitle="zoom4x";
   
 
   // printInteger((s64)zoom,0, 10, 255, 255, 255, SUB_FRAME1);
    //printInteger((s64)zoom,0, 10, 255, 255, 255, SUB_FRAME2);
   
    paint_word(title,104,5,128,128,255,0);
    
    printInteger(offx,240, 30, 10, 0, 0);
    printInteger(offy,240, 40, 10, 0, 0);
    printInteger(sx,240, 50, 10, 0, 0);
    printInteger(sy,240, 60, 10, 0, 0);
    printInteger(zoom,240, 70, 2, 0, 0);
    
  
    paint_word(zx,80,30,255,255,128,0);
    paint_word(zy,80,40,255,255,128,0);
    paint_word(px,80,50,255,255,128,0);
    paint_word(py,80,60,255,255,128,0);
    paint_word(ztitle,80,70,255,255,128,0);
    //drawBox(100,100,20,20,255,0,SUB_FRAME1);
    //drawBox(100,100,20,20,255,0,SUB_FRAME2);
};
